-- Top register for stack processor  Rob Chapman  Mar 2, 1998

-- A dual input register is used and the output of it is check to see when
-- it is not zero.

Library IEEE;
use IEEE.STD_Logic_1164.all;

entity Top is
  generic ( width : positive := 8 );
  port ( din1, din2          : in std_logic_vector(width-1 downto 0);
         d1_d2, clock, store : in std_logic;
         notzero             : out std_logic;
         dout                : out std_logic_vector(width-1 downto 0));
end;

architecture behaviour of Top is
  -- use a dual input register
  component Dir
    generic ( width : positive );
    port ( din1, din2          : in std_logic_vector(width-1 downto 0);
           d1_d2, clock, store : in std_logic;
           dout                : out std_logic_vector(width-1 downto 0));
  end component;

  signal data_out : std_logic_vector(width-1 downto 0);

begin

  -- dual input register
  DR : component Dir
    generic map ( width => width )
    port map ( din1 => din1, din2 => din2, d1_d2 => d1_d2, clock => clock,
               store => store, dout => data_out );

  dout <= data_out;

  -- test for notzero on the output
  process(data_out)
    variable bittest : std_logic;
  begin
    bittest := '0'; -- do a test for notzero in top
    for I in data_out'range loop
      bittest := bittest or data_out(I);
    end loop;
    notzero <= bittest;
  end process;

end behaviour;

